@@ -43,6 +43,7 @@
 #include "memoryagent.h"
 #include "moduleshandler.h"
 #include "registerhandler.h"
+#include "peripheralregisterhandler.h"
 #include "sourcefileshandler.h"
 #include "sourceutils.h"
 #include "stackhandler.h"
@@ -61,6 +62,7 @@
 #include <coreplugin/icore.h>
 #include <coreplugin/idocument.h>
 #include <coreplugin/messagebox.h>
+#include <coreplugin/modemanager.h>
 #include <coreplugin/progressmanager/progressmanager.h>
 #include <coreplugin/progressmanager/futureprogress.h>
 
@@ -72,7 +74,6 @@
 #include <texteditor/fontsettings.h>
 
 #include <utils/basetreeview.h>
-#include <utils/fileinprojectfinder.h>
 #include <utils/macroexpander.h>
 #include <utils/processhandle.h>
 #include <utils/qtcassert.h>
@@ -172,7 +173,7 @@ Location::Location(const StackFrame &frame, bool marker)
 }
 
 
-LocationMark::LocationMark(DebuggerEngine *engine, const FileName &file, int line)
+LocationMark::LocationMark(DebuggerEngine *engine, const FilePath &file, int line)
     : TextMark(file, line, Constants::TEXT_MARK_CATEGORY_LOCATION), m_engine(engine)
 {
     setPriority(TextMark::HighPriority);
@@ -224,7 +225,7 @@ public:
     {
         auto agent = new MemoryAgent(data, engine);
         if (agent->isUsable()) {
-            m_agents.append(agent);
+            m_agents.push_back(agent);
         } else {
             delete agent;
             AsynchronousMessageBox::warning(
@@ -237,7 +238,7 @@ public:
     // On stack frame completed and on request.
     void updateContents()
     {
-        foreach (MemoryAgent *agent, m_agents) {
+        for (MemoryAgent *agent : m_agents) {
             if (agent)
                 agent->updateContents();
         }
@@ -245,14 +246,14 @@ public:
 
     void handleDebuggerFinished()
     {
-        foreach (MemoryAgent *agent, m_agents) {
+        for (MemoryAgent *agent : m_agents) {
             if (agent)
                 agent->setFinished(); // Prevent triggering updates, etc.
         }
     }
 
 private:
-    QList<MemoryAgent *> m_agents;
+    std::vector<MemoryAgent *> m_agents;
 };
 
 
@@ -273,6 +274,7 @@ public:
           m_breakHandler(engine),
           m_modulesHandler(engine),
           m_registerHandler(engine),
+          m_peripheralRegisterHandler(engine),
           m_sourceFilesHandler(engine),
           m_stackHandler(engine),
           m_threadsHandler(engine),
@@ -280,12 +282,16 @@ public:
           m_disassemblerAgent(engine),
           m_toolTipManager(engine)
     {
-        m_logWindow = new LogWindow(m_engine); // Needed before start()
-        m_logWindow->setObjectName(QLatin1String(DOCKWIDGET_OUTPUT));
         m_debuggerName = DebuggerEngine::tr("Debugger");
 
-        connect(action(EnableReverseDebugging), &SavedAction::valueChanged,
-                this, [this] { updateState(true); });
+        m_logWindow = new LogWindow(m_engine); // Needed before start()
+        m_logWindow->setObjectName("Debugger.Dock.Output");
+
+        connect(action(EnableReverseDebugging), &SavedAction::valueChanged, this, [this] {
+            updateState();
+            if (m_companionEngine)
+                m_companionEngine->d->updateState();
+        });
         static int contextCount = 0;
         m_context = Context(Id("Debugger.Engine.").withSuffix(++contextCount));
 
@@ -338,6 +344,7 @@ public:
         delete m_watchersWindow;
         delete m_inspectorWindow;
         delete m_registerWindow;
+        delete m_peripheralRegisterWindow;
         delete m_modulesWindow;
         delete m_sourceFilesWindow;
         delete m_stackWindow;
@@ -349,6 +356,7 @@ public:
         delete m_watchersView;
         delete m_inspectorView;
         delete m_registerView;
+        delete m_peripheralRegisterView;
         delete m_modulesView;
         delete m_sourceFilesView;
         delete m_stackView;
@@ -370,17 +378,15 @@ public:
         if (!m_perspective)
             return;
 
-        delete m_perspective;
+        Perspective *perspective = m_perspective;
         m_perspective = nullptr;
 
         EngineManager::unregisterEngine(m_engine);
 
-        // Give up ownership on claimed breakpoints.
-        m_breakHandler.releaseAllBreakpoints();
-        m_toolTipManager.deregisterEngine();
-        m_memoryAgents.handleDebuggerFinished();
-
-        setBusyCursor(false);
+        // This triggers activity in the EngineManager which
+        // recognizes the rampdown by the m_perpective == nullptr above.
+        perspective->destroy();
+        delete perspective;
     }
 
     void updateReturnViewHeader(int section, int, int newSize)
@@ -418,6 +424,8 @@ public:
         m_watchHandler.cleanup();
         m_engine->showMessage(tr("Debugger finished."), StatusBar);
         m_engine->setState(DebuggerFinished); // Also destroys views.
+        if (boolSetting(SwitchModeOnExit))
+            EngineManager::deactivateDebugMode();
     }
 
     void scheduleResetLocation()
@@ -435,7 +443,6 @@ public:
         m_locationTimer.stop();
         m_locationMark.reset();
         m_stackHandler.resetLocation();
-        m_watchHandler.resetLocation();
         m_disassemblerAgent.resetLocation();
         m_toolTipManager.resetLocation();
     }
@@ -444,14 +451,13 @@ public:
     void setInitialActionStates();
     void setBusyCursor(bool on);
     void cleanupViews();
-    void updateState(bool alsoUpdateCompanion);
+    void updateState();
     void updateReverseActions();
 
     DebuggerEngine *m_engine = nullptr; // Not owned.
     QString m_runId;
-    QPointer<RunConfiguration> m_runConfiguration;  // Not owned.
     QString m_debuggerName;
-    Perspective *m_perspective = nullptr;
+    QPointer<Perspective> m_perspective;
     DebuggerRunParameters m_runParameters;
     IDevice::ConstPtr m_device;
 
@@ -467,6 +473,7 @@ public:
     BreakHandler m_breakHandler;
     ModulesHandler m_modulesHandler;
     RegisterHandler m_registerHandler;
+    PeripheralRegisterHandler m_peripheralRegisterHandler;
     SourceFilesHandler m_sourceFilesHandler;
     StackHandler m_stackHandler;
     ThreadsHandler m_threadsHandler;
@@ -478,7 +485,6 @@ public:
     QScopedPointer<LocationMark> m_locationMark;
     QTimer m_locationTimer;
 
-    Utils::FileInProjectFinder m_fileFinder;
     QString m_qtNamespace;
 
     // Safety net to avoid infinite lookups.
@@ -491,6 +497,7 @@ public:
     QPointer<BaseTreeView> m_watchersView;
     QPointer<WatchTreeView> m_inspectorView;
     QPointer<BaseTreeView> m_registerView;
+    QPointer<BaseTreeView> m_peripheralRegisterView;
     QPointer<BaseTreeView> m_modulesView;
     QPointer<BaseTreeView> m_sourceFilesView;
     QPointer<BaseTreeView> m_stackView;
@@ -501,6 +508,7 @@ public:
     QPointer<QWidget> m_watchersWindow;
     QPointer<QWidget> m_inspectorWindow;
     QPointer<QWidget> m_registerWindow;
+    QPointer<QWidget> m_peripheralRegisterWindow;
     QPointer<QWidget> m_modulesWindow;
     QPointer<QWidget> m_sourceFilesWindow;
     QPointer<QWidget> m_stackWindow;
@@ -513,11 +521,6 @@ public:
     bool m_busy = false;
     bool m_isDying = false;
 
-    QAction m_debugWithoutDeployAction;
-    QAction m_attachToQmlPortAction;
-    QAction m_attachToRemoteServerAction;
-    QAction m_startRemoteCdbAction;
-    QAction m_attachToCoreAction;
     QAction m_detachAction;
     OptionalAction m_continueAction{tr("Continue")};
     QAction m_exitAction{tr("Stop Debugger")}; // On application output button if "Stop" is possible
@@ -554,16 +557,17 @@ public:
 void DebuggerEnginePrivate::setupViews()
 {
     const DebuggerRunParameters &rp = m_runParameters;
+    const QString engineId = EngineManager::registerEngine(m_engine);
 
     QTC_CHECK(!m_perspective);
 
-    m_perspective = new Perspective("Debugger.Perspective." + m_runId,
+    const QString perspectiveId = "Debugger.Perspective." + m_runId + '.' + m_debuggerName;
+    const QString settingsId = "Debugger.Perspective." + m_debuggerName;
+
+    m_perspective = new Perspective(perspectiveId,
                                     m_engine->displayName(),
                                     Debugger::Constants::PRESET_PERSPECTIVE_ID,
-                                    m_debuggerName);
-    m_perspective->setShouldPersistChecker([this] {
-        return EngineManager::isLastOf(m_debuggerName);
-    });
+                                    settingsId);
 
     m_progress.setProgressRange(0, 1000);
     FutureProgress *fp = ProgressManager::addTask(m_progress.future(),
@@ -631,7 +635,7 @@ void DebuggerEnginePrivate::setupViews()
             m_engine, &DebuggerEngine::reloadModules,
             Qt::QueuedConnection);
     m_modulesWindow = addSearch(m_modulesView);
-    m_modulesWindow->setObjectName(DOCKWIDGET_MODULES);
+    m_modulesWindow->setObjectName("Debugger.Dock.Modules." + engineId);
     m_modulesWindow->setWindowTitle(tr("&Modules"));
 
     m_registerView = new BaseTreeView;
@@ -642,15 +646,26 @@ void DebuggerEnginePrivate::setupViews()
             m_engine, &DebuggerEngine::reloadRegisters,
             Qt::QueuedConnection);
     m_registerWindow = addSearch(m_registerView);
-    m_registerWindow->setObjectName(DOCKWIDGET_REGISTER);
+    m_registerWindow->setObjectName("Debugger.Dock.Register." + engineId);
     m_registerWindow->setWindowTitle(tr("Reg&isters"));
 
-    m_stackView = new BaseTreeView;
+    m_peripheralRegisterView = new BaseTreeView;
+    m_peripheralRegisterView->setModel(m_peripheralRegisterHandler.model());
+    m_peripheralRegisterView->setRootIsDecorated(true);
+    m_peripheralRegisterView->setSettings(settings, "Debugger.PeripheralRegisterView");
+    connect(m_peripheralRegisterView, &BaseTreeView::aboutToShow,
+            m_engine, &DebuggerEngine::reloadPeripheralRegisters,
+            Qt::QueuedConnection);
+    m_peripheralRegisterWindow = addSearch(m_peripheralRegisterView);
+    m_peripheralRegisterWindow->setObjectName("Debugger.Dock.PeripheralRegister." + engineId);
+    m_peripheralRegisterWindow->setWindowTitle(tr("Peripheral Reg&isters"));
+
+    m_stackView = new StackTreeView;
     m_stackView->setModel(m_stackHandler.model());
     m_stackView->setSettings(settings, "Debugger.StackView");
     m_stackView->setIconSize(QSize(10, 10));
     m_stackWindow = addSearch(m_stackView);
-    m_stackWindow->setObjectName(DOCKWIDGET_STACK);
+    m_stackWindow->setObjectName("Debugger.Dock.Stack." + engineId);
     m_stackWindow->setWindowTitle(tr("&Stack"));
 
     m_sourceFilesView = new BaseTreeView;
@@ -661,7 +676,7 @@ void DebuggerEnginePrivate::setupViews()
             m_engine, &DebuggerEngine::reloadSourceFiles,
             Qt::QueuedConnection);
     m_sourceFilesWindow = addSearch(m_sourceFilesView);
-    m_sourceFilesWindow->setObjectName(DOCKWIDGET_SOURCE_FILES);
+    m_sourceFilesWindow->setObjectName("Debugger.Dock.SourceFiles." + engineId);
     m_sourceFilesWindow->setWindowTitle(tr("Source Files"));
 
     m_threadsView = new BaseTreeView;
@@ -669,8 +684,9 @@ void DebuggerEnginePrivate::setupViews()
     m_threadsView->setSortingEnabled(true);
     m_threadsView->setSettings(settings, "Debugger.ThreadsView");
     m_threadsView->setIconSize(QSize(10, 10));
+    m_threadsView->setSpanColumn(ThreadData::FunctionColumn);
     m_threadsWindow = addSearch(m_threadsView);
-    m_threadsWindow->setObjectName(DOCKWIDGET_THREADS);
+    m_threadsWindow->setObjectName("Debugger.Dock.Threads." + engineId);
     m_threadsWindow->setWindowTitle(tr("&Threads"));
 
     m_returnView = new WatchTreeView{ReturnType};
@@ -684,26 +700,26 @@ void DebuggerEnginePrivate::setupViews()
     m_localsView->setModel(m_watchHandler.model());
     m_localsView->setSettings(settings, "Debugger.LocalsView");
     m_localsWindow = addSearch(m_localsView);
-    m_localsWindow->setObjectName("CppDebugLocals");
+    m_localsWindow->setObjectName("Debugger.Dock.Locals." + engineId);
     m_localsWindow->setWindowTitle(tr("Locals"));
 
     m_inspectorView = new WatchTreeView{InspectType};
     m_inspectorView->setModel(m_watchHandler.model());
     m_inspectorView->setSettings(settings, "Debugger.LocalsView"); // sic! same as locals view.
     m_inspectorWindow = addSearch(m_inspectorView);
-    m_inspectorWindow->setObjectName("Inspector");
+    m_inspectorWindow->setObjectName("Debugger.Dock.Inspector." + engineId);
     m_inspectorWindow->setWindowTitle(tr("Locals"));
 
     m_watchersView = new WatchTreeView{WatchersType};
     m_watchersView->setModel(m_watchHandler.model());
     m_watchersView->setSettings(settings, "Debugger.WatchersView");
     m_watchersWindow = addSearch(m_watchersView);
-    m_watchersWindow->setObjectName("CppDebugWatchers");
+    m_watchersWindow->setObjectName("Debugger.Dock.Watchers." + engineId);
     m_watchersWindow->setWindowTitle(tr("&Expressions"));
 
     m_localsAndInspectorWindow = new LocalsAndInspectorWindow(
                 m_localsWindow, m_inspectorWindow, m_returnWindow);
-    m_localsAndInspectorWindow->setObjectName(DOCKWIDGET_LOCALS_AND_INSPECTOR);
+    m_localsAndInspectorWindow->setObjectName("Debugger.Dock.LocalsAndInspector." + engineId);
     m_localsAndInspectorWindow->setWindowTitle(m_localsWindow->windowTitle());
 
     // Locals
@@ -714,13 +730,14 @@ void DebuggerEnginePrivate::setupViews()
     m_breakView->setIconSize(QSize(10, 10));
     m_breakView->setWindowIcon(Icons::BREAKPOINTS.icon());
     m_breakView->setSelectionMode(QAbstractItemView::ExtendedSelection);
+    m_breakView->setSpanColumn(BreakpointFunctionColumn);
     connect(action(UseAddressInBreakpointsView), &QAction::toggled,
             this, [this](bool on) { m_breakView->setColumnHidden(BreakpointAddressColumn, !on); });
     m_breakView->setSettings(settings, "Debugger.BreakWindow");
     m_breakView->setModel(m_breakHandler.model());
     m_breakView->setRootIsDecorated(true);
     m_breakWindow = addSearch(m_breakView);
-    m_breakWindow->setObjectName(DOCKWIDGET_BREAK);
+    m_breakWindow->setObjectName("Debugger.Dock.Break." + engineId);
     m_breakWindow->setWindowTitle(tr("&Breakpoints"));
 
     m_perspective->useSubPerspectiveSwitcher(EngineManager::engineChooser());
@@ -741,7 +758,7 @@ void DebuggerEnginePrivate::setupViews()
     connect(&m_continueAction, &QAction::triggered,
             m_engine, &DebuggerEngine::handleExecContinue);
 
-    m_exitAction.setIcon(Icons::DEBUG_EXIT_SMALL.icon());
+    m_exitAction.setIcon(Icons::DEBUG_EXIT_SMALL_TOOLBAR.icon());
     connect(&m_exitAction, &QAction::triggered,
             m_engine, &DebuggerEngine::requestRunControlStop);
 
@@ -816,6 +833,7 @@ void DebuggerEnginePrivate::setupViews()
         m_modulesWindow->setFont(font);
         //m_consoleWindow->setFont(font);
         m_registerWindow->setFont(font);
+        m_peripheralRegisterWindow->setFont(font);
         m_returnWindow->setFont(font);
         m_sourceFilesWindow->setFont(font);
         m_stackWindow->setFont(font);
@@ -832,6 +850,7 @@ void DebuggerEnginePrivate::setupViews()
     m_perspective->addWindow(m_localsAndInspectorWindow, Perspective::AddToTab, nullptr, true, Qt::RightDockWidgetArea);
     m_perspective->addWindow(m_watchersWindow, Perspective::SplitVertical, m_localsAndInspectorWindow, true, Qt::RightDockWidgetArea);
     m_perspective->addWindow(m_registerWindow, Perspective::AddToTab, m_localsAndInspectorWindow, false, Qt::RightDockWidgetArea);
+    m_perspective->addWindow(m_peripheralRegisterWindow, Perspective::AddToTab, m_localsAndInspectorWindow, false, Qt::RightDockWidgetArea);
     m_perspective->addWindow(m_logWindow, Perspective::AddToTab, nullptr, false, Qt::TopDockWidgetArea);
 
     m_perspective->select();
@@ -847,7 +866,6 @@ void DebuggerEnginePrivate::setupViews()
 DebuggerEngine::DebuggerEngine()
   : d(new DebuggerEnginePrivate(this))
 {
-    updateState(false);
 }
 
 DebuggerEngine::~DebuggerEngine()
@@ -910,6 +928,11 @@ bool DebuggerEngine::isRegistersWindowVisible() const
     return d->m_registerWindow->isVisible();
 }
 
+bool DebuggerEngine::isPeripheralRegistersWindowVisible() const
+{
+    return d->m_peripheralRegisterWindow->isVisible();
+}
+
 bool DebuggerEngine::isModulesWindowVisible() const
 {
     return d->m_modulesWindow->isVisible();
@@ -941,6 +964,11 @@ RegisterHandler *DebuggerEngine::registerHandler() const
     return &d->m_registerHandler;
 }
 
+PeripheralRegisterHandler *DebuggerEngine::peripheralRegisterHandler() const
+{
+    return &d->m_peripheralRegisterHandler;
+}
+
 StackHandler *DebuggerEngine::stackHandler() const
 {
     return &d->m_stackHandler;
@@ -978,20 +1006,26 @@ DisassemblerAgent *DebuggerEngine::disassemblerAgent() const
 
 void DebuggerEngine::fetchMemory(MemoryAgent *, quint64 addr, quint64 length)
 {
-    Q_UNUSED(addr);
-    Q_UNUSED(length);
+    Q_UNUSED(addr)
+    Q_UNUSED(length)
 }
 
 void DebuggerEngine::changeMemory(MemoryAgent *, quint64 addr, const QByteArray &data)
 {
-    Q_UNUSED(addr);
-    Q_UNUSED(data);
+    Q_UNUSED(addr)
+    Q_UNUSED(data)
 }
 
 void DebuggerEngine::setRegisterValue(const QString &name, const QString &value)
 {
-    Q_UNUSED(name);
-    Q_UNUSED(value);
+    Q_UNUSED(name)
+    Q_UNUSED(value)
+}
+
+void DebuggerEngine::setPeripheralRegisterValue(quint64 address, quint64 value)
+{
+    Q_UNUSED(address)
+    Q_UNUSED(value)
 }
 
 void DebuggerEngine::setRunParameters(const DebuggerRunParameters &runParameters)
@@ -1008,7 +1042,6 @@ void DebuggerEngine::setRunId(const QString &id)
 void DebuggerEngine::setRunTool(DebuggerRunTool *runTool)
 {
     RunControl *runControl = runTool->runControl();
-    d->m_runConfiguration = runControl->runConfiguration();
     d->m_device = runControl->device();
     if (!d->m_device)
         d->m_device = d->m_runParameters.inferior.device;
@@ -1021,7 +1054,6 @@ void DebuggerEngine::setRunTool(DebuggerRunTool *runTool)
 
 void DebuggerEngine::start()
 {
-    EngineManager::registerEngine(this);
     d->m_watchHandler.resetWatchers();
     d->setInitialActionStates();
     setState(EngineSetupRequested);
@@ -1066,7 +1098,7 @@ void DebuggerEngine::gotoLocation(const Location &loc)
         editor->document()->setProperty(Constants::OPENED_BY_DEBUGGER, true);
 
     if (loc.needsMarker()) {
-        d->m_locationMark.reset(new LocationMark(this, FileName::fromString(file), line));
+        d->m_locationMark.reset(new LocationMark(this, FilePath::fromString(file), line));
         d->m_locationMark->setToolTip(tr("Current debugger location of %1").arg(displayName()));
     }
 }
@@ -1117,7 +1149,7 @@ void DebuggerEngine::abortDebugger()
 
 void DebuggerEngine::updateUi(bool isCurrentEngine)
 {
-    updateState(false);
+    updateState();
     if (isCurrentEngine) {
         gotoCurrentLocation();
     } else {
@@ -1320,7 +1352,7 @@ void DebuggerEngine::notifyInferiorSpontaneousStop()
 {
     showMessage("NOTE: INFERIOR SPONTANEOUS STOP");
     QTC_ASSERT(state() == InferiorRunOk, qDebug() << this << state());
-    EngineManager::activateEngine(this);
+    d->m_perspective->select();
     showMessage(tr("Stopped."), StatusBar);
     setState(InferiorStopOk);
     if (boolSetting(RaiseOnInterrupt))
@@ -1357,9 +1389,6 @@ void DebuggerEnginePrivate::setInitialActionStates()
 
     m_snapshotAction.setIcon(Utils::Icons::SNAPSHOT_TOOLBAR.icon());
 
-    m_attachToQmlPortAction.setEnabled(true);
-    m_attachToCoreAction.setEnabled(true);
-    m_attachToRemoteServerAction.setEnabled(true);
     m_detachAction.setEnabled(false);
 
     m_watchAction.setEnabled(true);
@@ -1390,10 +1419,12 @@ void DebuggerEnginePrivate::setInitialActionStates()
     m_threadLabel->setEnabled(false);
 }
 
-void DebuggerEnginePrivate::updateState(bool alsoUpdateCompanion)
+void DebuggerEnginePrivate::updateState()
 {
-    if (!m_perspective)
+    // Can happen in mixed debugging.
+    if (!m_threadLabel)
         return;
+    QTC_ASSERT(m_threadLabel, return);
 
     const DebuggerState state = m_state;
     const bool companionPreventsAction = m_engine->companionPreventsActions();
@@ -1403,7 +1434,7 @@ void DebuggerEnginePrivate::updateState(bool alsoUpdateCompanion)
     // visible, possibly disabled.
     if (state == DebuggerNotReady) {
         // Happens when companion starts, otherwise this should not happen.
-        QTC_CHECK(m_companionEngine);
+        //QTC_CHECK(m_companionEngine);
         m_interruptAction.setVisible(true);
         m_interruptAction.setEnabled(false);
         m_continueAction.setVisible(false);
@@ -1412,7 +1443,6 @@ void DebuggerEnginePrivate::updateState(bool alsoUpdateCompanion)
         m_stepIntoAction.setEnabled(true);
         m_stepOutAction.setEnabled(false);
         m_exitAction.setEnabled(false);
-        m_debugWithoutDeployAction.setEnabled(true);
     } else if (state == InferiorStopOk) {
         // F5 continues, Shift-F5 kills. It is "continuable".
         m_interruptAction.setVisible(false);
@@ -1423,7 +1453,6 @@ void DebuggerEnginePrivate::updateState(bool alsoUpdateCompanion)
         m_stepIntoAction.setEnabled(!companionPreventsAction);
         m_stepOutAction.setEnabled(!companionPreventsAction);
         m_exitAction.setEnabled(true);
-        m_debugWithoutDeployAction.setEnabled(false);
         m_localsAndInspectorWindow->setShowLocals(true);
     } else if (state == InferiorRunOk) {
         // Shift-F5 interrupts. It is also "interruptible".
@@ -1435,10 +1464,8 @@ void DebuggerEnginePrivate::updateState(bool alsoUpdateCompanion)
         m_stepIntoAction.setEnabled(false);
         m_stepOutAction.setEnabled(false);
         m_exitAction.setEnabled(true);
-        m_debugWithoutDeployAction.setEnabled(false);
         m_localsAndInspectorWindow->setShowLocals(false);
     } else if (state == DebuggerFinished) {
-        const bool canRun = ProjectExplorerPlugin::canRunStartupProject(ProjectExplorer::Constants::DEBUG_RUN_MODE);
         // We don't want to do anything anymore.
         m_interruptAction.setVisible(true);
         m_interruptAction.setEnabled(false);
@@ -1448,7 +1475,6 @@ void DebuggerEnginePrivate::updateState(bool alsoUpdateCompanion)
         m_stepIntoAction.setEnabled(false);
         m_stepOutAction.setEnabled(false);
         m_exitAction.setEnabled(false);
-        m_debugWithoutDeployAction.setEnabled(canRun);
         setBusyCursor(false);
         cleanupViews();
     } else if (state == InferiorUnrunnable) {
@@ -1461,7 +1487,6 @@ void DebuggerEnginePrivate::updateState(bool alsoUpdateCompanion)
         m_stepIntoAction.setEnabled(false);
         m_stepOutAction.setEnabled(false);
         m_exitAction.setEnabled(true);
-        m_debugWithoutDeployAction.setEnabled(false);
         // show locals in core dumps
         m_localsAndInspectorWindow->setShowLocals(true);
     } else {
@@ -1474,13 +1499,8 @@ void DebuggerEnginePrivate::updateState(bool alsoUpdateCompanion)
         m_stepIntoAction.setEnabled(false);
         m_stepOutAction.setEnabled(false);
         m_exitAction.setEnabled(false);
-        m_debugWithoutDeployAction.setEnabled(false);
     }
 
-    m_attachToQmlPortAction.setEnabled(true);
-    m_attachToCoreAction.setEnabled(true);
-    m_attachToRemoteServerAction.setEnabled(true);
-
     const bool threadsEnabled = state == InferiorStopOk || state == InferiorUnrunnable;
     m_threadsHandler.threadSwitcher()->setEnabled(threadsEnabled);
     m_threadLabel->setEnabled(threadsEnabled);
@@ -1544,9 +1564,6 @@ void DebuggerEnginePrivate::updateState(bool alsoUpdateCompanion)
         || state == DebuggerFinished
         || state == InferiorUnrunnable;
     setBusyCursor(!notbusy);
-
-    if (alsoUpdateCompanion && m_companionEngine)
-        m_companionEngine->updateState(false);
 }
 
 void DebuggerEnginePrivate::updateReverseActions()
@@ -1611,6 +1628,7 @@ void DebuggerEnginePrivate::setBusyCursor(bool busy)
     m_modulesWindow->setCursor(cursor);
     m_logWindow->setCursor(cursor);
     m_registerWindow->setCursor(cursor);
+    m_peripheralRegisterWindow->setCursor(cursor);
     m_returnWindow->setCursor(cursor);
     m_sourceFilesWindow->setCursor(cursor);
     m_stackWindow->setCursor(cursor);
@@ -1705,9 +1723,9 @@ void DebuggerEngine::notifyInferiorExited()
     d->doShutdownEngine();
 }
 
-void DebuggerEngine::updateState(bool alsoUpdateCompanion)
+void DebuggerEngine::updateState()
 {
-    d->updateState(alsoUpdateCompanion);
+    d->updateState();
 }
 
 WatchTreeView *DebuggerEngine::inspectorView()
@@ -1723,7 +1741,7 @@ void DebuggerEngine::showMessage(const QString &msg, int channel, int timeout) c
         case StatusBar:
             d->m_logWindow->showInput(LogMisc, msg);
             d->m_logWindow->showOutput(LogMisc, msg);
-            Debugger::showStatusMessage(msg, timeout);
+            DebuggerMainWindow::showStatusMessage(msg, timeout);
             break;
         case LogMiscInput:
             d->m_logWindow->showInput(LogMisc, msg);
@@ -1734,8 +1752,8 @@ void DebuggerEngine::showMessage(const QString &msg, int channel, int timeout) c
             d->m_logWindow->showOutput(LogInput, msg);
             break;
         case LogError:
-            d->m_logWindow->showInput(LogError, QLatin1String("ERROR: ") + msg);
-            d->m_logWindow->showOutput(LogError, QLatin1String("ERROR: ") + msg);
+            d->m_logWindow->showInput(LogError, "ERROR: " + msg);
+            d->m_logWindow->showOutput(LogError, "ERROR: " + msg);
             break;
         case AppOutput:
         case AppStuff:
@@ -1808,17 +1826,28 @@ void DebuggerEngine::setState(DebuggerState state, bool forced)
     if (!forced && !isAllowedTransition(oldState, state))
         qDebug() << "*** UNEXPECTED STATE TRANSITION: " << this << msg;
 
-    if (state == EngineRunRequested)
+    if (state == EngineRunRequested) {
         emit engineStarted();
+        d->m_perspective->select();
+    }
 
     showMessage(msg, LogDebug);
 
-    d->updateState(true);
+    d->updateState();
+    if (d->m_companionEngine)
+        d->m_companionEngine->d->updateState();
 
     if (oldState != d->m_state)
         emit EngineManager::instance()->engineStateChanged(this);
 
     if (state == DebuggerFinished) {
+        d->setBusyCursor(false);
+
+        // Give up ownership on claimed breakpoints.
+        d->m_breakHandler.releaseAllBreakpoints();
+        d->m_toolTipManager.deregisterEngine();
+        d->m_memoryAgents.handleDebuggerFinished();
+
         d->destroyPerspective();
         emit engineFinished();
     }
@@ -1834,18 +1863,6 @@ bool DebuggerEngine::canDisplayTooltip() const
     return state() == InferiorStopOk;
 }
 
-QString DebuggerEngine::toFileInProject(const QUrl &fileUrl)
-{
-    // make sure file finder is properly initialized
-    const DebuggerRunParameters &rp = runParameters();
-    d->m_fileFinder.setProjectDirectory(rp.projectSourceDirectory);
-    d->m_fileFinder.setProjectFiles(rp.projectSourceFiles);
-    d->m_fileFinder.setAdditionalSearchDirectories(rp.additionalSearchDirectories);
-    d->m_fileFinder.setSysroot(rp.sysRoot);
-
-    return d->m_fileFinder.findFile(fileUrl);
-}
-
 QString DebuggerEngine::expand(const QString &string) const
 {
     return runParameters().macroExpander->expand(string);
@@ -1894,7 +1911,7 @@ void DebuggerEngine::operateByInstructionTriggered(bool on)
 {
     // Go to source only if we have the file.
     //    if (DebuggerEngine *cppEngine = m_engine->cppEngine()) {
-    d->m_stackHandler.resetModel();
+    d->m_stackHandler.rootItem()->updateAll();
     if (d->m_stackHandler.currentIndex() >= 0) {
         const StackFrame frame = d->m_stackHandler.currentFrame();
         if (on || frame.isUsable())
@@ -2001,11 +2018,6 @@ void DebuggerEngine::progressPing()
     d->m_progress.setProgressValue(progress);
 }
 
-bool DebuggerEngine::isStartupRunConfiguration() const
-{
-    return d->m_runConfiguration == RunConfiguration::startupRunConfiguration();
-}
-
 void DebuggerEngine::setCompanionEngine(DebuggerEngine *engine)
 {
     d->m_companionEngine = engine;
@@ -2087,6 +2099,10 @@ void DebuggerEngine::reloadRegisters()
 {
 }
 
+void DebuggerEngine::reloadPeripheralRegisters()
+{
+}
+
 void DebuggerEngine::reloadSourceFiles()
 {
 }
@@ -2270,8 +2286,8 @@ void DebuggerEngine::openDisassemblerView(const Location &location)
 
 void DebuggerEngine::raiseWatchersWindow()
 {
-    if (d->m_watchersView) {
-        if (auto dock = qobject_cast<QDockWidget *>(d->m_watchersView->parentWidget())) {
+    if (d->m_watchersView && d->m_watchersWindow) {
+        if (auto dock = qobject_cast<QDockWidget *>(d->m_watchersWindow->parentWidget())) {
             if (QAction *act = dock->toggleViewAction()) {
                 if (!act->isChecked())
                     QTimer::singleShot(1, act, [act] { act->trigger(); });
@@ -2444,7 +2460,7 @@ void DebuggerEngine::handleExecRunToSelectedFunction()
     if (functionName.isEmpty()) {
         const QTextBlock block = cursor.block();
         const QString line = block.text();
-        foreach (const QString &str, line.trimmed().split(QLatin1Char('('))) {
+        foreach (const QString &str, line.trimmed().split('(')) {
             QString a;
             for (int i = str.size(); --i >= 0; ) {
                 if (!str.at(i).isLetterOrNumber())
@@ -2532,7 +2548,7 @@ bool DebuggerEngine::isNativeMixedActiveFrame() const
 {
     if (!isNativeMixedActive())
         return false;
-    if (stackHandler()->frames().isEmpty())
+    if (stackHandler()->rowCount() == 0)
         return false;
     StackFrame frame = stackHandler()->frameAt(0);
     return frame.language == QmlLanguage;
@@ -2541,8 +2557,6 @@ bool DebuggerEngine::isNativeMixedActiveFrame() const
 void DebuggerEngine::startDying() const
 {
     d->m_isDying = true;
-    if (DebuggerEngine *other = d->m_companionEngine)
-        other->d->m_isDying = true;
 }
 
 QString DebuggerEngine::runId() const
@@ -2576,8 +2590,7 @@ QString DebuggerEngine::formatStartParameters() const
         str << "qml";
     str << '\n';
     if (!sp.inferior.executable.isEmpty()) {
-        str << "Executable: " << QDir::toNativeSeparators(sp.inferior.executable)
-            << ' ' << sp.inferior.commandLineArguments;
+        str << "Executable: " << sp.inferior.commandLine().toUserOutput();
         if (d->m_terminalRunner)
             str << " [terminal]";
         str << '\n';
@@ -2585,9 +2598,8 @@ QString DebuggerEngine::formatStartParameters() const
             str << "Directory: " << QDir::toNativeSeparators(sp.inferior.workingDirectory)
                 << '\n';
     }
-    QString cmd = sp.debugger.executable;
-    if (!cmd.isEmpty())
-        str << "Debugger: " << QDir::toNativeSeparators(cmd) << '\n';
+    if (!sp.debugger.executable.isEmpty())
+        str << "Debugger: " << sp.debugger.executable.toUserOutput() << '\n';
     if (!sp.coreFile.isEmpty())
         str << "Core: " << QDir::toNativeSeparators(sp.coreFile) << '\n';
     if (sp.attachPID.isValid())
@@ -2595,7 +2607,7 @@ QString DebuggerEngine::formatStartParameters() const
     if (!sp.projectSourceDirectory.isEmpty()) {
         str << "Project: " << sp.projectSourceDirectory.toUserOutput() << '\n';
         str << "Additional Search Directories:";
-        for (const FileName &dir : sp.additionalSearchDirectories)
+        for (const FilePath &dir : sp.additionalSearchDirectories)
             str << ' ' << dir;
         str << '\n';
     }
@@ -2604,10 +2616,77 @@ QString DebuggerEngine::formatStartParameters() const
     if (!sp.qmlServer.host().isEmpty())
         str << "QML server: " << sp.qmlServer.host() << ':' << sp.qmlServer.port() << '\n';
     str << "Sysroot: " << sp.sysRoot << '\n';
-    str << "Debug Source Location: " << sp.debugSourceLocation.join(QLatin1Char(':')) << '\n';
+    str << "Debug Source Location: " << sp.debugSourceLocation.join(':') << '\n';
     return rc;
 }
 
+static void createNewDock(QWidget *widget)
+{
+    auto dockWidget = new QDockWidget;
+    dockWidget->setWidget(widget);
+    dockWidget->setWindowTitle(widget->windowTitle());
+    dockWidget->setFeatures(QDockWidget::DockWidgetClosable);
+    dockWidget->show();
+}
+
+void DebuggerEngine::showModuleSymbols(const QString &moduleName, const Symbols &symbols)
+{
+    auto w = new QTreeWidget;
+    w->setUniformRowHeights(true);
+    w->setColumnCount(5);
+    w->setRootIsDecorated(false);
+    w->setAlternatingRowColors(true);
+    w->setSortingEnabled(true);
+    w->setObjectName("Symbols." + moduleName);
+    QStringList header;
+    header.append(tr("Symbol"));
+    header.append(tr("Address"));
+    header.append(tr("Code"));
+    header.append(tr("Section"));
+    header.append(tr("Name"));
+    w->setHeaderLabels(header);
+    w->setWindowTitle(tr("Symbols in \"%1\"").arg(moduleName));
+    for (const Symbol &s : symbols) {
+        auto it = new QTreeWidgetItem;
+        it->setData(0, Qt::DisplayRole, s.name);
+        it->setData(1, Qt::DisplayRole, s.address);
+        it->setData(2, Qt::DisplayRole, s.state);
+        it->setData(3, Qt::DisplayRole, s.section);
+        it->setData(4, Qt::DisplayRole, s.demangled);
+        w->addTopLevelItem(it);
+    }
+    createNewDock(w);
+}
+
+void DebuggerEngine::showModuleSections(const QString &moduleName, const Sections &sections)
+{
+    auto w = new QTreeWidget;
+    w->setUniformRowHeights(true);
+    w->setColumnCount(5);
+    w->setRootIsDecorated(false);
+    w->setAlternatingRowColors(true);
+    w->setSortingEnabled(true);
+    w->setObjectName("Sections." + moduleName);
+    QStringList header;
+    header.append(tr("Name"));
+    header.append(tr("From"));
+    header.append(tr("To"));
+    header.append(tr("Address"));
+    header.append(tr("Flags"));
+    w->setHeaderLabels(header);
+    w->setWindowTitle(tr("Sections in \"%1\"").arg(moduleName));
+    for (const Section &s : sections) {
+        auto it = new QTreeWidgetItem;
+        it->setData(0, Qt::DisplayRole, s.name);
+        it->setData(1, Qt::DisplayRole, s.from);
+        it->setData(2, Qt::DisplayRole, s.to);
+        it->setData(3, Qt::DisplayRole, s.address);
+        it->setData(4, Qt::DisplayRole, s.flags);
+        w->addTopLevelItem(it);
+    }
+    createNewDock(w);
+}
+
 // CppDebuggerEngine
 
 Context CppDebuggerEngine::languageContext() const
@@ -2638,12 +2717,15 @@ void CppDebuggerEngine::validateRunParameters(DebuggerRunParameters &rp)
                         "experience for this binary format.").arg(preferredDebugger);
             break;
         }
-        if (warnOnRelease && rp.cppEngineType == CdbEngineType) {
-            if (!rp.symbolFile.endsWith(".exe", Qt::CaseInsensitive))
-                rp.symbolFile.append(".exe");
+        if (warnOnRelease
+                && rp.cppEngineType == CdbEngineType
+                && rp.startMode != AttachToRemoteServer) {
+            QTC_ASSERT(!rp.symbolFile.isEmpty(), return);
+            if (!rp.symbolFile.exists() && !rp.symbolFile.endsWith(".exe"))
+                rp.symbolFile = rp.symbolFile.stringAppended(".exe");
             QString errorMessage;
             QStringList rc;
-            if (getPDBFiles(rp.symbolFile, &rc, &errorMessage) && !rc.isEmpty())
+            if (getPDBFiles(rp.symbolFile.toString(), &rc, &errorMessage) && !rc.isEmpty())
                 return;
             if (!errorMessage.isEmpty()) {
                 detailedWarning.append('\n');
@@ -2664,11 +2746,11 @@ void CppDebuggerEngine::validateRunParameters(DebuggerRunParameters &rp)
             break;
         }
 
-        Utils::ElfReader reader(rp.symbolFile);
+        Utils::ElfReader reader(rp.symbolFile.toString());
         const ElfData elfData = reader.readHeaders();
         const QString error = reader.errorString();
 
-        showMessage("EXAMINING " + rp.symbolFile, LogDebug);
+        showMessage("EXAMINING " + rp.symbolFile.toString(), LogDebug);
         QByteArray msg = "ELF SECTIONS: ";
 
         static const QList<QByteArray> interesting = {
@@ -2711,7 +2793,7 @@ void CppDebuggerEngine::validateRunParameters(DebuggerRunParameters &rp)
             QSharedPointer<GlobalDebuggerOptions> options = Internal::globalDebuggerOptions();
             SourcePathRegExpMap globalRegExpSourceMap;
             globalRegExpSourceMap.reserve(options->sourcePathRegExpMap.size());
-            foreach (auto entry, options->sourcePathRegExpMap) {
+            for (auto entry : qAsConst(options->sourcePathRegExpMap)) {
                 const QString expanded = Utils::globalMacroExpander()->expand(entry.second);
                 if (!expanded.isEmpty())
                     globalRegExpSourceMap.push_back(qMakePair(entry.first, expanded));
@@ -2748,7 +2830,7 @@ void CppDebuggerEngine::validateRunParameters(DebuggerRunParameters &rp)
         if (hasEmbeddedInfo || hasLink)
             return;
 
-        foreach (const QByteArray &name, interesting) {
+        for (const QByteArray &name : qAsConst(interesting)) {
             const QString found = seen.contains(name) ? DebuggerEngine::tr("Found.")
                                                       : DebuggerEngine::tr("Not found.");
             detailedWarning.append('\n' + DebuggerEngine::tr("Section %1: %2").arg(QString::fromUtf8(name)).arg(found));
