@@ -32,14 +32,12 @@
 
 #include <coreplugin/icore.h>
 #include <coreplugin/coreicons.h>
-#include <extensionsystem/pluginmanager.h>
+
 #include <utils/qtcassert.h>
 #include <utils/detailswidget.h>
 #include <utils/hostosinfo.h>
 #include <utils/theme/theme.h>
 
-#include <QSignalMapper>
-
 #include <QLabel>
 #include <QPushButton>
 #include <QMenu>
@@ -52,11 +50,10 @@ using namespace ProjectExplorer;
 using namespace ProjectExplorer::Internal;
 using namespace Utils;
 
-ToolWidget::ToolWidget(QWidget *parent) : FadingPanel(parent),
-    m_targetOpacity(1.0f)
+ToolWidget::ToolWidget(QWidget *parent) : FadingPanel(parent)
 {
     auto layout = new QHBoxLayout;
-    layout->setMargin(4);
+    layout->setContentsMargins(4, 4, 4, 4);
     layout->setSpacing(4);
     setLayout(layout);
     m_firstWidget = new FadingWidget(this);
@@ -69,7 +66,6 @@ ToolWidget::ToolWidget(QWidget *parent) : FadingPanel(parent),
 
     m_disableButton = new QToolButton(m_firstWidget);
     m_disableButton->setAutoRaise(true);
-    m_disableButton->setToolTip(BuildStepListWidget::tr("Disable"));
     m_disableButton->setFixedSize(buttonSize);
     m_disableButton->setIcon(Icons::BUILDSTEP_DISABLE.icon());
     m_disableButton->setCheckable(true);
@@ -79,7 +75,7 @@ ToolWidget::ToolWidget(QWidget *parent) : FadingPanel(parent),
     m_secondWidget = new FadingWidget(this);
     m_secondWidget->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);
     hbox = new QHBoxLayout();
-    hbox->setMargin(0);
+    hbox->setContentsMargins(0, 0, 0, 0);
     hbox->setSpacing(4);
     m_secondWidget->setLayout(hbox);
 
@@ -138,11 +134,13 @@ void ToolWidget::setBuildStepEnabled(bool b)
             m_firstWidget->fadeTo(m_targetOpacity);
     } else {
         if (HostOsInfo::isMacHost())
-            m_firstWidget->setOpacity(1.0);
+            m_firstWidget->setOpacity(.999);
         else
-            m_firstWidget->fadeTo(1.0);
+            m_firstWidget->fadeTo(.999);
     }
     m_disableButton->setChecked(!b);
+    m_disableButton->setToolTip(b ? BuildStepListWidget::tr("Disable")
+                                  : BuildStepListWidget::tr("Enable"));
 }
 
 void ToolWidget::setUpEnabled(bool b)
@@ -171,7 +169,7 @@ void ToolWidget::setDownVisible(bool b)
 }
 
 BuildStepsWidgetData::BuildStepsWidgetData(BuildStep *s) :
-    step(s), widget(0), detailsWidget(0)
+    step(s), widget(nullptr), detailsWidget(nullptr)
 {
     widget = s->createConfigWidget();
     Q_ASSERT(widget);
@@ -185,7 +183,6 @@ BuildStepsWidgetData::BuildStepsWidgetData(BuildStep *s) :
     detailsWidget->setToolWidget(toolWidget);
     detailsWidget->setContentsMargins(0, 0, 0, 1);
     detailsWidget->setSummaryText(widget->summaryText());
-    detailsWidget->setAdditionalSummaryText(widget->additionalSummaryText());
 }
 
 BuildStepsWidgetData::~BuildStepsWidgetData()
@@ -195,11 +192,8 @@ BuildStepsWidgetData::~BuildStepsWidgetData()
 }
 
 BuildStepListWidget::BuildStepListWidget(QWidget *parent) :
-    NamedWidget(parent),
-    m_buildStepList(0),
-    m_addButton(0)
-{
-}
+    NamedWidget(parent)
+{ }
 
 BuildStepListWidget::~BuildStepListWidget()
 {
@@ -220,19 +214,6 @@ void BuildStepListWidget::updateSummary()
     }
 }
 
-void BuildStepListWidget::updateAdditionalSummary()
-{
-    auto widget = qobject_cast<BuildStepConfigWidget *>(sender());
-    if (widget) {
-        foreach (const BuildStepsWidgetData *s, m_buildStepsData) {
-            if (s->widget == widget) {
-                s->detailsWidget->setAdditionalSummaryText(widget->additionalSummaryText());
-                break;
-            }
-        }
-    }
-}
-
 void BuildStepListWidget::updateEnabledState()
 {
     auto step = qobject_cast<BuildStep *>(sender());
@@ -277,8 +258,11 @@ void BuildStepListWidget::init(BuildStepList *bsl)
     for (int i = 0; i < bsl->count(); ++i) {
         addBuildStep(i);
         // addBuilStep expands the config widget by default, which we don't want here
-        if (m_buildStepsData.at(i)->widget->showWidget())
-            m_buildStepsData.at(i)->detailsWidget->setState(DetailsWidget::Collapsed);
+        if (m_buildStepsData.at(i)->step->widgetExpandedByDefault()) {
+            m_buildStepsData.at(i)->detailsWidget->setState(
+                        m_buildStepsData.at(i)->step->wasUserExpanded()
+                        ? DetailsWidget::Expanded : DetailsWidget::Collapsed);
+        }
     }
 
     m_noStepsLabel->setVisible(bsl->isEmpty());
@@ -291,12 +275,11 @@ void BuildStepListWidget::init(BuildStepList *bsl)
 
 void BuildStepListWidget::updateAddBuildStepMenu()
 {
-    QMap<QString, QPair<Core::Id, IBuildStepFactory *> > map;
+    QMap<QString, QPair<Core::Id, BuildStepFactory *> > map;
     //Build up a list of possible steps and save map the display names to the (internal) name and factories.
-    QList<IBuildStepFactory *> factories = ExtensionSystem::PluginManager::getObjects<IBuildStepFactory>();
-    foreach (IBuildStepFactory *factory, factories) {
-        const QList<BuildStepInfo> infos = factory->availableSteps(m_buildStepList);
-        for (const BuildStepInfo &info : infos) {
+    for (BuildStepFactory *factory : BuildStepFactory::allBuildStepFactories()) {
+        if (factory->canHandle(m_buildStepList)) {
+            const BuildStepInfo &info = factory->stepInfo();
             if (info.flags & BuildStepInfo::Uncreatable)
                 continue;
             if ((info.flags & BuildStepInfo::UniqueStep) && m_buildStepList->contains(info.id))
@@ -309,11 +292,11 @@ void BuildStepListWidget::updateAddBuildStepMenu()
     QMenu *menu = m_addButton->menu();
     menu->clear();
     if (!map.isEmpty()) {
-        QMap<QString, QPair<Core::Id, IBuildStepFactory *> >::const_iterator it, end;
+        QMap<QString, QPair<Core::Id, BuildStepFactory *> >::const_iterator it, end;
         end = map.constEnd();
         for (it = map.constBegin(); it != end; ++it) {
             QAction *action = menu->addAction(it.key());
-            IBuildStepFactory *factory = it.value().second;
+            BuildStepFactory *factory = it.value().second;
             Core::Id id = it.value().first;
 
             connect(action, &QAction::triggered, [id, factory, this]() {
@@ -326,52 +309,32 @@ void BuildStepListWidget::updateAddBuildStepMenu()
     }
 }
 
-void BuildStepListWidget::addBuildStepWidget(int pos, BuildStep *step)
+void BuildStepListWidget::addBuildStep(int pos)
 {
+    BuildStep *newStep = m_buildStepList->at(pos);
+
     // create everything
-    auto s = new BuildStepsWidgetData(step);
+    auto s = new BuildStepsWidgetData(newStep);
     m_buildStepsData.insert(pos, s);
 
     m_vbox->insertWidget(pos, s->detailsWidget);
 
     connect(s->widget, &BuildStepConfigWidget::updateSummary,
             this, &BuildStepListWidget::updateSummary);
-    connect(s->widget, &BuildStepConfigWidget::updateAdditionalSummary,
-            this, &BuildStepListWidget::updateAdditionalSummary);
 
     connect(s->step, &BuildStep::enabledChanged,
             this, &BuildStepListWidget::updateEnabledState);
 
-    connect(s->toolWidget, &ToolWidget::disabledClicked,
-            m_disableMapper, static_cast<void (QSignalMapper::*)()>(&QSignalMapper::map));
-    connect(s->toolWidget, &ToolWidget::upClicked,
-            m_upMapper, static_cast<void (QSignalMapper::*)()>(&QSignalMapper::map));
-    connect(s->toolWidget, &ToolWidget::downClicked,
-            m_downMapper, static_cast<void (QSignalMapper::*)()>(&QSignalMapper::map));
-    connect(s->toolWidget, &ToolWidget::removeClicked,
-            m_removeMapper, static_cast<void (QSignalMapper::*)()>(&QSignalMapper::map));
-}
-
-void BuildStepListWidget::addBuildStep(int pos)
-{
-    BuildStep *newStep = m_buildStepList->at(pos);
-    addBuildStepWidget(pos, newStep);
-    BuildStepsWidgetData *s = m_buildStepsData.at(pos);
     // Expand new build steps by default
-    if (s->widget->showWidget())
-        s->detailsWidget->setState(DetailsWidget::Expanded);
-    else
-        s->detailsWidget->setState(DetailsWidget::OnlySummary);
+    const bool expand = newStep->hasUserExpansionState()
+            ? newStep->wasUserExpanded() : newStep->widgetExpandedByDefault();
+    s->detailsWidget->setState(expand ? DetailsWidget::Expanded : DetailsWidget::OnlySummary);
+    connect(s->detailsWidget, &DetailsWidget::expanded, newStep, &BuildStep::setUserExpanded);
 
     m_noStepsLabel->setVisible(false);
     updateBuildStepButtonsState();
 }
 
-void BuildStepListWidget::triggerStepMoveUp(int pos)
-{
-    m_buildStepList->moveStepUp(pos);
-}
-
 void BuildStepListWidget::stepMoved(int from, int to)
 {
     m_vbox->insertWidget(to, m_buildStepsData.at(from)->detailsWidget);
@@ -383,21 +346,6 @@ void BuildStepListWidget::stepMoved(int from, int to)
     updateBuildStepButtonsState();
 }
 
-void BuildStepListWidget::triggerStepMoveDown(int pos)
-{
-    triggerStepMoveUp(pos + 1);
-}
-
-void BuildStepListWidget::triggerRemoveBuildStep(int pos)
-{
-    if (!m_buildStepList->removeStep(pos)) {
-        QMessageBox::warning(Core::ICore::mainWindow(),
-                             tr("Removing Step failed"),
-                             tr("Cannot remove build step while building"),
-                             QMessageBox::Ok, QMessageBox::Ok);
-    }
-}
-
 void BuildStepListWidget::removeBuildStep(int pos)
 {
     delete m_buildStepsData.takeAt(pos);
@@ -408,31 +356,11 @@ void BuildStepListWidget::removeBuildStep(int pos)
     m_noStepsLabel->setVisible(hasSteps);
 }
 
-void BuildStepListWidget::triggerDisable(int pos)
-{
-    BuildStep *bs = m_buildStepsData.at(pos)->step;
-    bs->setEnabled(!bs->enabled());
-    m_buildStepsData.at(pos)->toolWidget->setBuildStepEnabled(bs->enabled());
-}
-
 void BuildStepListWidget::setupUi()
 {
     if (m_addButton)
         return;
 
-    m_disableMapper = new QSignalMapper(this);
-    connect(m_disableMapper, static_cast<void (QSignalMapper::*)(int)>(&QSignalMapper::mapped),
-            this, &BuildStepListWidget::triggerDisable);
-    m_upMapper = new QSignalMapper(this);
-    connect(m_upMapper, static_cast<void (QSignalMapper::*)(int)>(&QSignalMapper::mapped),
-            this, &BuildStepListWidget::triggerStepMoveUp);
-    m_downMapper = new QSignalMapper(this);
-    connect(m_downMapper, static_cast<void (QSignalMapper::*)(int)>(&QSignalMapper::mapped),
-            this, &BuildStepListWidget::triggerStepMoveDown);
-    m_removeMapper = new QSignalMapper(this);
-    connect(m_removeMapper, static_cast<void (QSignalMapper::*)(int)>(&QSignalMapper::mapped),
-            this, &BuildStepListWidget::triggerRemoveBuildStep);
-
     m_vbox = new QVBoxLayout(this);
     m_vbox->setContentsMargins(0, 0, 0, 0);
     m_vbox->setSpacing(0);
@@ -464,38 +392,37 @@ void BuildStepListWidget::updateBuildStepButtonsState()
         return;
     for (int i = 0; i < m_buildStepsData.count(); ++i) {
         BuildStepsWidgetData *s = m_buildStepsData.at(i);
-        m_disableMapper->setMapping(s->toolWidget, i);
-        s->toolWidget->setRemoveEnabled(!m_buildStepList->at(i)->immutable());
-        m_removeMapper->setMapping(s->toolWidget, i);
+        disconnect(s->toolWidget, nullptr, this, nullptr);
+        connect(s->toolWidget, &ToolWidget::disabledClicked,
+                this, [s] {
+            BuildStep *bs = s->step;
+            bs->setEnabled(!bs->enabled());
+            s->toolWidget->setBuildStepEnabled(bs->enabled());
+        });
+        s->toolWidget->setRemoveEnabled(!m_buildStepList->at(i)->isImmutable());
+        connect(s->toolWidget, &ToolWidget::removeClicked,
+                this, [this, i] {
+            if (!m_buildStepList->removeStep(i)) {
+                QMessageBox::warning(Core::ICore::mainWindow(),
+                                     tr("Removing Step failed"),
+                                     tr("Cannot remove build step while building"),
+                                     QMessageBox::Ok, QMessageBox::Ok);
+            }
+        });
 
         s->toolWidget->setUpEnabled((i > 0)
-                                    && !(m_buildStepList->at(i)->immutable()
-                                         && m_buildStepList->at(i - 1)->immutable()));
-        m_upMapper->setMapping(s->toolWidget, i);
+                                    && !(m_buildStepList->at(i)->isImmutable()
+                                         && m_buildStepList->at(i - 1)->isImmutable()));
+        connect(s->toolWidget, &ToolWidget::upClicked,
+                this, [this, i] { m_buildStepList->moveStepUp(i); });
         s->toolWidget->setDownEnabled((i + 1 < m_buildStepList->count())
-                                      && !(m_buildStepList->at(i)->immutable()
-                                           && m_buildStepList->at(i + 1)->immutable()));
-        m_downMapper->setMapping(s->toolWidget, i);
+                                      && !(m_buildStepList->at(i)->isImmutable()
+                                           && m_buildStepList->at(i + 1)->isImmutable()));
+        connect(s->toolWidget, &ToolWidget::downClicked,
+                this, [this, i] { m_buildStepList->moveStepUp(i + 1); });
 
         // Only show buttons when needed
         s->toolWidget->setDownVisible(m_buildStepList->count() != 1);
         s->toolWidget->setUpVisible(m_buildStepList->count() != 1);
     }
 }
-
-BuildStepsPage::BuildStepsPage(BuildConfiguration *bc, Core::Id id) :
-    m_id(id),
-    m_widget(new BuildStepListWidget(this))
-{
-    auto layout = new QVBoxLayout(this);
-    layout->setMargin(0);
-    layout->setSpacing(0);
-    layout->addWidget(m_widget);
-
-    m_widget->init(bc->stepList(m_id));
-
-    if (m_id == Constants::BUILDSTEPS_BUILD)
-        setDisplayName(tr("Build Steps"));
-    if (m_id == Constants::BUILDSTEPS_CLEAN)
-        setDisplayName(tr("Clean Steps"));
-}
