@@ -52,6 +52,8 @@
 #include <QVBoxLayout>
 #include <QCheckBox>
 
+using namespace Utils;
+
 namespace ProjectExplorer {
 
 static QList<IPotentialKit *> g_potentialKits;
@@ -67,10 +69,10 @@ IPotentialKit::~IPotentialKit()
 }
 
 namespace Internal {
-static Utils::FileName importDirectory(const QString &projectPath)
+static FilePath importDirectory(const FilePath &projectPath)
 {
     // Setup import widget:
-    auto path = Utils::FileName::fromString(projectPath);
+    auto path = projectPath;
     path = path.parentDir(); // base dir
     path = path.parentDir(); // parent dir
 
@@ -84,18 +86,13 @@ public:
     QWidget *scrollAreaWidget;
     QScrollArea *scrollArea;
     QLabel *headerLabel;
-    QLabel *descriptionLabel;
     QLabel *noValidKitLabel;
-    QLabel *optionHintLabel;
     QCheckBox *allKitsCheckBox;
-    Utils::FancyLineEdit *kitFilterLineEdit;
+    FancyLineEdit *kitFilterLineEdit;
 
     void setupUi(TargetSetupPage *q)
     {
         auto setupTargetPage = new QWidget(q);
-        descriptionLabel = new QLabel(setupTargetPage);
-        descriptionLabel->setWordWrap(true);
-        descriptionLabel->setVisible(false);
 
         headerLabel = new QLabel(setupTargetPage);
         headerLabel->setWordWrap(true);
@@ -103,22 +100,17 @@ public:
 
         noValidKitLabel = new QLabel(setupTargetPage);
         noValidKitLabel->setWordWrap(true);
-        noValidKitLabel->setText(TargetSetupPage::tr("<span style=\" font-weight:600;\">No valid kits found.</span>"));
-
-
-        optionHintLabel = new QLabel(setupTargetPage);
-        optionHintLabel->setWordWrap(true);
-        optionHintLabel->setText(TargetSetupPage::tr(
-                                     "Please add a kit in the <a href=\"buildandrun\">options</a> "
-                                     "or via the maintenance tool of the SDK."));
-        optionHintLabel->setTextInteractionFlags(Qt::TextBrowserInteraction);
-        optionHintLabel->setVisible(false);
+        noValidKitLabel->setText(TargetSetupPage::tr("<span style=\" font-weight:600;\">No suitable kits found.</span><br/>"
+                                 "Add a kit in the <a href=\"buildandrun\">options</a> "
+                                 "or via the maintenance tool of the SDK."));
+        noValidKitLabel->setTextInteractionFlags(Qt::TextBrowserInteraction);
+        noValidKitLabel->setVisible(false);
 
         allKitsCheckBox = new QCheckBox(setupTargetPage);
         allKitsCheckBox->setTristate(true);
         allKitsCheckBox->setText(TargetSetupPage::tr("Select all kits"));
 
-        kitFilterLineEdit = new Utils::FancyLineEdit(setupTargetPage);
+        kitFilterLineEdit = new FancyLineEdit(setupTargetPage);
         kitFilterLineEdit->setFiltering(true);
         kitFilterLineEdit->setPlaceholderText(TargetSetupPage::tr("Type to filter kits by name..."));
 
@@ -144,10 +136,8 @@ public:
 
         auto verticalLayout_2 = new QVBoxLayout(setupTargetPage);
         verticalLayout_2->addWidget(headerLabel);
-        verticalLayout_2->addWidget(descriptionLabel);
         verticalLayout_2->addWidget(kitFilterLineEdit);
         verticalLayout_2->addWidget(noValidKitLabel);
-        verticalLayout_2->addWidget(optionHintLabel);
         verticalLayout_2->addWidget(allKitsCheckBox);
         verticalLayout_2->addWidget(centralWidget);
         verticalLayout_2->addWidget(scrollAreaWidget);
@@ -156,13 +146,13 @@ public:
         verticalLayout_3->setContentsMargins(0, 0, 0, -1);
         verticalLayout_3->addWidget(setupTargetPage);
 
-        QObject::connect(optionHintLabel, &QLabel::linkActivated,
+        QObject::connect(noValidKitLabel, &QLabel::linkActivated,
                          q, &TargetSetupPage::openOptions);
 
         QObject::connect(allKitsCheckBox, &QAbstractButton::clicked,
                          q, &TargetSetupPage::changeAllKitsSelections);
 
-        QObject::connect(kitFilterLineEdit, &Utils::FancyLineEdit::filterChanged,
+        QObject::connect(kitFilterLineEdit, &FancyLineEdit::filterChanged,
                          q, &TargetSetupPage::kitFilterChanged);
     }
 };
@@ -172,7 +162,7 @@ public:
 using namespace Internal;
 
 TargetSetupPage::TargetSetupPage(QWidget *parent) :
-    Utils::WizardPage(parent),
+    WizardPage(parent),
     m_ui(new TargetSetupPageUi),
     m_importWidget(new ImportWidget(this)),
     m_spacer(new QSpacerItem(0,0, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding))
@@ -193,7 +183,7 @@ TargetSetupPage::TargetSetupPage(QWidget *parent) :
     m_ui->scrollArea->setWidget(centralWidget);
     centralWidget->setLayout(new QVBoxLayout);
     m_ui->centralWidget->setLayout(new QVBoxLayout);
-    m_ui->centralWidget->layout()->setMargin(0);
+    m_ui->centralWidget->layout()->setContentsMargins(0, 0, 0, 0);
 
     setTitle(tr("Kit Selection"));
 
@@ -210,18 +200,21 @@ TargetSetupPage::TargetSetupPage(QWidget *parent) :
     connect(km, &KitManager::kitRemoved, this, &TargetSetupPage::handleKitRemoval);
     connect(km, &KitManager::kitUpdated, this, &TargetSetupPage::handleKitUpdate);
     connect(m_importWidget, &ImportWidget::importFrom,
-            this, [this](const Utils::FileName &dir) { import(dir); });
+            this, [this](const FilePath &dir) { import(dir); });
+    connect(KitManager::instance(), &KitManager::kitsChanged,
+            this, &TargetSetupPage::updateVisibility);
 
-    setProperty(Utils::SHORT_TITLE_PROPERTY, tr("Kits"));
+    setProperty(SHORT_TITLE_PROPERTY, tr("Kits"));
 }
 
 void TargetSetupPage::initializePage()
 {
-    reset();
-
-    setupWidgets();
-    setupImports();
-    selectAtLeastOneKit();
+    if (KitManager::isLoaded()) {
+        doInitializePage();
+    } else {
+        connect(KitManager::instance(), &KitManager::kitsLoaded,
+                this, &TargetSetupPage::doInitializePage);
+    }
 }
 
 void TargetSetupPage::setRequiredKitPredicate(const Kit::Predicate &predicate)
@@ -248,35 +241,43 @@ TargetSetupPage::~TargetSetupPage()
 {
     disconnect();
     reset();
+    delete m_spacer;
     delete m_ui;
 }
 
 bool TargetSetupPage::isComplete() const
 {
-    return Utils::anyOf(m_widgets, [](const TargetSetupWidget *w) {
+    return anyOf(m_widgets, [](const TargetSetupWidget *w) {
         return w->isKitSelected();
     });
 }
 
 void TargetSetupPage::setupWidgets(const QString &filterText)
 {
-    // Known profiles:
-    auto kitList = sortedKitList(m_requiredPredicate);
-
-    foreach (Kit *k, kitList) {
-        if (filterText.isEmpty() || k->displayName().contains(filterText, Qt::CaseInsensitive))
-            addWidget(k);
+    const auto kitList = KitManager::sortKits(KitManager::kits());
+    for (Kit *k : kitList) {
+        if (!filterText.isEmpty() && !k->displayName().contains(filterText, Qt::CaseInsensitive))
+            continue;
+        const auto widget = new TargetSetupWidget(k, m_projectPath);
+        setInitialCheckState(widget);
+        connect(widget, &TargetSetupWidget::selectedToggled,
+                this, &TargetSetupPage::kitSelectionChanged);
+        connect(widget, &TargetSetupWidget::selectedToggled, this, &QWizardPage::completeChanged);
+        updateWidget(widget);
+        m_widgets.push_back(widget);
+        m_baseLayout->addWidget(widget);
     }
+    addAdditionalWidgets();
 
     // Setup import widget:
     m_importWidget->setCurrentDirectory(Internal::importDirectory(m_projectPath));
 
     updateVisibility();
-    selectAtLeastOneKit();
 }
 
 void TargetSetupPage::reset()
 {
+    removeAdditionalWidgets();
     while (m_widgets.size() > 0) {
         TargetSetupWidget *w = m_widgets.back();
 
@@ -290,19 +291,25 @@ void TargetSetupPage::reset()
     m_ui->allKitsCheckBox->setChecked(false);
 }
 
+void TargetSetupPage::setInitialCheckState(TargetSetupWidget *widget)
+{
+    widget->setKitSelected(widget->isEnabled() && m_preferredPredicate
+                           && m_preferredPredicate(widget->kit()));
+}
+
 TargetSetupWidget *TargetSetupPage::widget(const Core::Id kitId,
                                            TargetSetupWidget *fallback) const
 {
-    return Utils::findOr(m_widgets, fallback, [kitId](const TargetSetupWidget *w) {
+    return findOr(m_widgets, fallback, [kitId](const TargetSetupWidget *w) {
         return w->kit() && w->kit()->id() == kitId;
     });
 }
 
-void TargetSetupPage::setProjectPath(const QString &path)
+void TargetSetupPage::setProjectPath(const FilePath &path)
 {
     m_projectPath = path;
     if (!m_projectPath.isEmpty()) {
-        QFileInfo fileInfo(QDir::cleanPath(path));
+        QFileInfo fileInfo(QDir::cleanPath(path.toString()));
         QStringList subDirsList = fileInfo.absolutePath().split('/');
         m_ui->headerLabel->setText(tr("The following kits can be used for project <b>%1</b>:",
                                       "%1: Project name").arg(subDirsList.last()));
@@ -328,16 +335,9 @@ void TargetSetupPage::setProjectImporter(ProjectImporter *importer)
         initializePage();
 }
 
-void TargetSetupPage::setNoteText(const QString &text)
+bool TargetSetupPage::importLineEditHasFocus() const
 {
-    m_ui->descriptionLabel->setText(text);
-    m_ui->descriptionLabel->setVisible(!text.isEmpty());
-}
-
-void TargetSetupPage::showOptionsHint(bool show)
-{
-    m_forceOptionHint = show;
-    updateVisibility();
+    return m_importWidget->ownsReturnKey();
 }
 
 void TargetSetupPage::setupImports()
@@ -345,9 +345,9 @@ void TargetSetupPage::setupImports()
     if (!m_importer || m_projectPath.isEmpty())
         return;
 
-    QStringList toImport = m_importer->importCandidates();
-    foreach (const QString &path, toImport)
-        import(Utils::FileName::fromString(path), true);
+    const QStringList toImport = m_importer->importCandidates();
+    for (const QString &path : toImport)
+        import(FilePath::fromString(path), true);
 }
 
 void TargetSetupPage::handleKitAddition(Kit *k)
@@ -381,35 +381,43 @@ void TargetSetupPage::handleKitUpdate(Kit *k)
     if (m_importer)
         m_importer->makePersistent(k);
 
-    bool acceptable = !m_requiredPredicate || m_requiredPredicate(k);
-
-    if (!acceptable)
-        removeWidget(k);
-    else if (acceptable)
-        addWidget(k);
-
+    const auto newWidgetList = sortedWidgetList();
+    if (newWidgetList != m_widgets) { // Sorting has changed.
+        m_widgets = newWidgetList;
+        reLayout();
+    }
+    updateWidget(widget(k));
     kitSelectionChanged();
     updateVisibility();
 }
 
 void TargetSetupPage::selectAtLeastOneKit()
 {
-    const bool atLeastOneKitSelected = Utils::anyOf(m_widgets, [](TargetSetupWidget *w) {
+    bool atLeastOneKitSelected = anyOf(m_widgets, [](TargetSetupWidget *w) {
             return w->isKitSelected();
     });
 
     if (!atLeastOneKitSelected) {
-        TargetSetupWidget *w = m_firstWidget;
-        Kit *defaultKit = KitManager::defaultKit();
-        if (defaultKit)
-            w = widget(defaultKit->id(), m_firstWidget);
-        if (w) {
-            w->setKitSelected(true);
-            kitSelectionChanged();
+        Kit * const defaultKit = KitManager::defaultKit();
+        if (defaultKit && isUsable(defaultKit)) {
+            if (TargetSetupWidget * const w = widget(defaultKit)) {
+                w->setKitSelected(true);
+                atLeastOneKitSelected = true;
+            }
         }
-        m_firstWidget = nullptr;
     }
-    emit completeChanged(); // Is this necessary?
+    if (!atLeastOneKitSelected) {
+        for (TargetSetupWidget * const w : qAsConst(m_widgets)) {
+            if (isUsable(w->kit())) {
+                w->setKitSelected(true);
+                atLeastOneKitSelected = true;
+            }
+        }
+    }
+    if (atLeastOneKitSelected) {
+        kitSelectionChanged();
+        emit completeChanged(); // Is this necessary?
+    }
 }
 
 void TargetSetupPage::updateVisibility()
@@ -418,14 +426,43 @@ void TargetSetupPage::updateVisibility()
     m_ui->scrollAreaWidget->setVisible(m_baseLayout == m_ui->scrollArea->widget()->layout());
     m_ui->centralWidget->setVisible(m_baseLayout == m_ui->centralWidget->layout());
 
-    bool hasKits = m_widgets.size() > 0;
-    m_ui->noValidKitLabel->setVisible(!hasKits);
-    m_ui->optionHintLabel->setVisible(m_forceOptionHint || !hasKits);
-    m_ui->allKitsCheckBox->setVisible(hasKits);
+    const bool hasUsableKits = KitManager::kit([this](const Kit *k) { return isUsable(k); });
+    m_ui->noValidKitLabel->setVisible(!hasUsableKits);
+    m_ui->allKitsCheckBox->setVisible(hasUsableKits);
 
     emit completeChanged();
 }
 
+void TargetSetupPage::reLayout()
+{
+    removeAdditionalWidgets();
+    for (TargetSetupWidget * const w : qAsConst(m_widgets))
+        m_baseLayout->removeWidget(w);
+    for (TargetSetupWidget * const w : qAsConst(m_widgets))
+        m_baseLayout->addWidget(w);
+    addAdditionalWidgets();
+}
+
+bool TargetSetupPage::compareKits(const Kit *k1, const Kit *k2)
+{
+    const QString name1 = k1->displayName();
+    const QString name2 = k2->displayName();
+    if (name1 < name2)
+        return true;
+    if (name1 > name2)
+        return false;
+    return k1 < k2;
+}
+
+std::vector<TargetSetupWidget *> TargetSetupPage::sortedWidgetList() const
+{
+    std::vector<TargetSetupWidget *> list = m_widgets;
+    sort(list, [](const TargetSetupWidget *w1, const TargetSetupWidget *w2) {
+        return compareKits(w1->kit(), w2->kit());
+    });
+    return list;
+}
+
 void TargetSetupPage::openOptions()
 {
     Core::ICore::showOptionsDialog(Constants::KITS_SETTINGS_PAGE_ID, this);
@@ -449,18 +486,21 @@ void TargetSetupPage::kitSelectionChanged()
         m_ui->allKitsCheckBox->setCheckState(Qt::Unchecked);
 }
 
-QList<Kit *> TargetSetupPage::sortedKitList(const Kit::Predicate &predicate)
-{
-    const auto kitList = KitManager::kits(predicate);
-
-    return KitManager::sortKits(kitList);
-}
-
 void TargetSetupPage::kitFilterChanged(const QString &filterText)
 {
     // Reset currently shown kits
     reset();
     setupWidgets(filterText);
+    selectAtLeastOneKit();
+}
+
+void TargetSetupPage::doInitializePage()
+{
+    reset();
+    setupWidgets();
+    setupImports();
+    selectAtLeastOneKit();
+    updateVisibility();
 }
 
 void TargetSetupPage::changeAllKitsSelections()
@@ -475,29 +515,24 @@ void TargetSetupPage::changeAllKitsSelections()
 
 bool TargetSetupPage::isUpdating() const
 {
-    if (m_importer)
-        return m_importer->isUpdating();
-    return false;
+    return m_importer && m_importer->isUpdating();
 }
 
-void TargetSetupPage::import(const Utils::FileName &path, bool silent)
+void TargetSetupPage::import(const FilePath &path, bool silent)
 {
     if (!m_importer)
         return;
 
-    QList<BuildInfo *> toImport = m_importer->import(path, silent);
-    foreach (BuildInfo *info, toImport) {
-        TargetSetupWidget *w = widget(info->kitId);
+    for (const BuildInfo &info : m_importer->import(path, silent)) {
+        TargetSetupWidget *w = widget(info.kitId);
         if (!w) {
-            Kit *k = KitManager::kit(info->kitId);
+            Kit *k = KitManager::kit(info.kitId);
             Q_ASSERT(k);
             addWidget(k);
         }
-        w = widget(info->kitId);
-        if (!w) {
-            delete info;
+        w = widget(info.kitId);
+        if (!w)
             continue;
-        }
 
         w->addBuildInfo(info, true);
         w->setKitSelected(true);
@@ -511,8 +546,6 @@ void TargetSetupPage::removeWidget(TargetSetupWidget *w)
 {
     if (!w)
         return;
-    if (w == m_firstWidget)
-        m_firstWidget = nullptr;
     w->deleteLater();
     w->clearKit();
     m_widgets.erase(std::find(m_widgets.begin(), m_widgets.end(), w));
@@ -520,50 +553,61 @@ void TargetSetupPage::removeWidget(TargetSetupWidget *w)
 
 TargetSetupWidget *TargetSetupPage::addWidget(Kit *k)
 {
-    if (!k || (m_requiredPredicate && !m_requiredPredicate(k)))
-        return nullptr;
-
-    const IBuildConfigurationFactory *const factory
-            = IBuildConfigurationFactory::find(k, m_projectPath);
-    const QList<BuildInfo *> infoList = [this, k, factory]() {
-        if (factory)
-            return factory->availableSetups(k, m_projectPath);
-
-        BuildInfo *info = new BuildInfo(nullptr);
-        info->kitId = k->id();
-        return QList<BuildInfo *>({info});
-    }();
-
-    // Not all projects have BuildConfigurations, that is perfectly fine.
-    TargetSetupWidget *widget = new TargetSetupWidget(k, m_projectPath);
-
-    m_baseLayout->removeWidget(m_importWidget);
-    foreach (QWidget *potentialWidget, m_potentialWidgets)
-        m_baseLayout->removeWidget(potentialWidget);
-    m_baseLayout->removeItem(m_spacer);
-
-    widget->setKitSelected(m_preferredPredicate && m_preferredPredicate(k));
-    m_widgets.push_back(widget);
+    const auto widget = new TargetSetupWidget(k, m_projectPath);
+    setInitialCheckState(widget);
+    updateWidget(widget);
     connect(widget, &TargetSetupWidget::selectedToggled,
             this, &TargetSetupPage::kitSelectionChanged);
-    m_baseLayout->addWidget(widget);
-
-    m_baseLayout->addWidget(m_importWidget);
-    foreach (QWidget *widget, m_potentialWidgets)
-        m_baseLayout->addWidget(widget);
-    m_baseLayout->addItem(m_spacer);
-
     connect(widget, &TargetSetupWidget::selectedToggled, this, &QWizardPage::completeChanged);
 
-    if (!m_firstWidget)
-        m_firstWidget = widget;
 
+    // Insert widget, sorted.
+    const auto insertionPos = std::find_if(m_widgets.begin(), m_widgets.end(),
+                                           [k](const TargetSetupWidget *w) {
+        return compareKits(k, w->kit());
+    });
+    const bool addedToEnd = insertionPos == m_widgets.end();
+    m_widgets.insert(insertionPos, widget);
+    if (addedToEnd) {
+        removeAdditionalWidgets();
+        m_baseLayout->addWidget(widget);
+        addAdditionalWidgets();
+    } else {
+        reLayout();
+    }
     return widget;
 }
 
+void TargetSetupPage::addAdditionalWidgets()
+{
+    m_baseLayout->addWidget(m_importWidget);
+    for (QWidget * const widget : qAsConst(m_potentialWidgets))
+        m_baseLayout->addWidget(widget);
+    m_baseLayout->addItem(m_spacer);
+}
+
+void TargetSetupPage::removeAdditionalWidgets(QLayout *layout)
+{
+    layout->removeWidget(m_importWidget);
+    for (QWidget * const potentialWidget : qAsConst(m_potentialWidgets))
+        layout->removeWidget(potentialWidget);
+    layout->removeItem(m_spacer);
+}
+
+void TargetSetupPage::updateWidget(TargetSetupWidget *widget)
+{
+    QTC_ASSERT(widget, return );
+    widget->update(m_requiredPredicate);
+}
+
+bool TargetSetupPage::isUsable(const Kit *kit) const
+{
+    return kit->isValid() && (!m_requiredPredicate || m_requiredPredicate(kit));
+}
+
 bool TargetSetupPage::setupProject(Project *project)
 {
-    QList<const BuildInfo *> toSetUp; // Pointers are managed by the widgets!
+    QList<BuildInfo> toSetUp;
     for (TargetSetupWidget *widget : m_widgets) {
         if (!widget->isKitSelected())
             continue;
@@ -600,17 +644,9 @@ void TargetSetupPage::setUseScrollArea(bool b)
     m_ui->scrollAreaWidget->setVisible(b);
     m_ui->centralWidget->setVisible(!b);
 
-    if (oldBaseLayout) {
-        oldBaseLayout->removeWidget(m_importWidget);
-        foreach (QWidget *widget, m_potentialWidgets)
-            oldBaseLayout->removeWidget(widget);
-        oldBaseLayout->removeItem(m_spacer);
-    }
-
-    m_baseLayout->addWidget(m_importWidget);
-    foreach (QWidget *widget, m_potentialWidgets)
-        m_baseLayout->addWidget(widget);
-    m_baseLayout->addItem(m_spacer);
+    if (oldBaseLayout)
+        removeAdditionalWidgets(oldBaseLayout);
+    addAdditionalWidgets();
 }
 
 } // namespace ProjectExplorer
