@@ -78,21 +78,21 @@ namespace QmakeProjectManager {
 // Helpers:
 // --------------------------------------------------------------------
 
-QString QmakeBuildConfiguration::shadowBuildDirectory(const QString &proFilePath, const Kit *k,
+QString QmakeBuildConfiguration::shadowBuildDirectory(const FilePath &proFilePath, const Kit *k,
                                                       const QString &suffix,
                                                       BuildConfiguration::BuildType buildType)
 {
     if (proFilePath.isEmpty())
         return QString();
 
-    const QString projectName = QFileInfo(proFilePath).completeBaseName();
+    const QString projectName = proFilePath.toFileInfo().completeBaseName();
     ProjectMacroExpander expander(proFilePath, projectName, k, suffix, buildType);
-    QString projectDir = Project::projectDirectory(FilePath::fromString(proFilePath)).toString();
+    QString projectDir = Project::projectDirectory(proFilePath).toString();
     QString buildPath = expander.expand(ProjectExplorerPlugin::buildDirectoryTemplate());
     return FileUtils::resolvePath(projectDir, buildPath);
 }
 
-static FilePath defaultBuildDirectory(const QString &projectPath,
+static FilePath defaultBuildDirectory(const FilePath &projectPath,
                                       const Kit *k,
                                       const QString &suffix,
                                       BuildConfiguration::BuildType type)
@@ -101,7 +101,6 @@ static FilePath defaultBuildDirectory(const QString &projectPath,
                                                                               suffix, type));
 }
 
-const char USE_SHADOW_BUILD_KEY[] = "Qt4ProjectManager.Qt4BuildConfiguration.UseShadowBuild";
 const char BUILD_CONFIGURATION_KEY[] = "Qt4ProjectManager.Qt4BuildConfiguration.BuildConfiguration";
 
 enum { debug = 0 };
@@ -128,23 +127,23 @@ QmakeBuildConfiguration::QmakeBuildConfiguration(Target *target, Core::Id id)
             this, &QmakeBuildConfiguration::qtVersionsChanged);
 }
 
-void QmakeBuildConfiguration::initialize(const BuildInfo &info)
+void QmakeBuildConfiguration::initialize()
 {
-    BuildConfiguration::initialize(info);
+    BuildConfiguration::initialize();
 
     BuildStepList *buildSteps = stepList(ProjectExplorer::Constants::BUILDSTEPS_BUILD);
     auto qmakeStep = new QMakeStep(buildSteps);
     buildSteps->appendStep(qmakeStep);
-    buildSteps->appendStep(new QmakeMakeStep(buildSteps));
+    buildSteps->appendStep(Constants::MAKESTEP_BS_ID);
 
     BuildStepList *cleanSteps = stepList(ProjectExplorer::Constants::BUILDSTEPS_CLEAN);
-    cleanSteps->appendStep(new QmakeMakeStep(cleanSteps));
+    cleanSteps->appendStep(Constants::MAKESTEP_BS_ID);
 
-    const QmakeExtraBuildInfo qmakeExtra = info.extraInfo.value<QmakeExtraBuildInfo>();
+    const QmakeExtraBuildInfo qmakeExtra = extraInfo().value<QmakeExtraBuildInfo>();
     BaseQtVersion *version = QtKitAspect::qtVersion(target()->kit());
 
     BaseQtVersion::QmakeBuildConfigs config = version->defaultBuildConfig();
-    if (info.buildType == BuildConfiguration::Debug)
+    if (initialBuildType() == BuildConfiguration::Debug)
         config |= BaseQtVersion::DebugBuild;
     else
         config &= ~BaseQtVersion::DebugBuild;
@@ -158,10 +157,11 @@ void QmakeBuildConfiguration::initialize(const BuildInfo &info)
 
     setQMakeBuildConfiguration(config);
 
-    FilePath directory = info.buildDirectory;
+    FilePath directory = initialBuildDirectory();
     if (directory.isEmpty()) {
-        directory = defaultBuildDirectory(target()->project()->projectFilePath().toString(),
-                                          target()->kit(), info.displayName, buildType());
+        directory = defaultBuildDirectory(target()->project()->projectFilePath(),
+                                          target()->kit(), initialDisplayName(),
+                                          initialBuildType());
     }
 
     setBuildDirectory(directory);
@@ -180,7 +180,6 @@ QmakeBuildConfiguration::~QmakeBuildConfiguration() = default;
 QVariantMap QmakeBuildConfiguration::toMap() const
 {
     QVariantMap map(BuildConfiguration::toMap());
-    map.insert(QLatin1String(USE_SHADOW_BUILD_KEY), m_shadowBuild);
     map.insert(QLatin1String(BUILD_CONFIGURATION_KEY), int(m_qmakeBuildConfiguration));
     return map;
 }
@@ -190,7 +189,6 @@ bool QmakeBuildConfiguration::fromMap(const QVariantMap &map)
     if (!BuildConfiguration::fromMap(map))
         return false;
 
-    m_shadowBuild = map.value(QLatin1String(USE_SHADOW_BUILD_KEY), true).toBool();
     m_qmakeBuildConfiguration = BaseQtVersion::QmakeBuildConfigs(map.value(QLatin1String(BUILD_CONFIGURATION_KEY)).toInt());
 
     m_lastKitState = LastKitState(target()->kit());
@@ -255,15 +253,6 @@ void QmakeBuildConfiguration::setFileNodeBuild(FileNode *node)
     m_fileNodeBuild = node;
 }
 
-/// returns whether this is a shadow build configuration or not
-/// note, even if shadowBuild() returns true, it might be using the
-/// source directory as the shadow build directory, thus it
-/// still is a in-source build
-bool QmakeBuildConfiguration::isShadowBuild() const
-{
-    return buildDirectory() != target()->project()->projectDirectory();
-}
-
 QString QmakeBuildConfiguration::makefile() const
 {
     auto rootNode = dynamic_cast<QmakeProFileNode *>(target()->project()->rootProjectNode());
@@ -311,11 +300,6 @@ bool QmakeBuildConfiguration::isBuildDirAtSafeLocation() const
                                     buildDirectory().toString());
 }
 
-void QmakeBuildConfiguration::emitQMakeBuildConfigurationChanged()
-{
-    emit qmakeBuildConfigurationChanged();
-}
-
 QStringList QmakeBuildConfiguration::configCommandLineArguments() const
 {
     QStringList result;
@@ -523,8 +507,8 @@ QString QmakeBuildConfiguration::extractSpecFromArguments(QString *args,
     if (parsedSpec.isEmpty())
         return {};
 
-    FilePath baseMkspecDir = FilePath::fromUserInput(
-            version->qmakeProperty("QT_HOST_DATA") + QLatin1String("/mkspecs"));
+    FilePath baseMkspecDir = FilePath::fromUserInput(version->hostDataPath().toString()
+                                                     + "/mkspecs");
     baseMkspecDir = FilePath::fromString(baseMkspecDir.toFileInfo().canonicalFilePath());
 
     // if the path is relative it can be
@@ -556,26 +540,6 @@ QString QmakeBuildConfiguration::extractSpecFromArguments(QString *args,
     return parsedSpec.toString();
 }
 
-bool QmakeBuildConfiguration::isEnabled() const
-{
-    return m_isEnabled;
-}
-
-QString QmakeBuildConfiguration::disabledReason() const
-{
-    if (!m_isEnabled)
-        return tr("Parsing the .pro file");
-    return QString();
-}
-
-void QmakeBuildConfiguration::setEnabled(bool enabled)
-{
-    if (m_isEnabled == enabled)
-        return;
-    m_isEnabled = enabled;
-    emit enabledChanged();
-}
-
 /*!
   \class QmakeBuildConfigurationFactory
 */
@@ -602,7 +566,7 @@ QmakeBuildConfigurationFactory::QmakeBuildConfigurationFactory()
 }
 
 BuildInfo QmakeBuildConfigurationFactory::createBuildInfo(const Kit *k,
-                                                          const QString &projectPath,
+                                                          const FilePath &projectPath,
                                                           BuildConfiguration::BuildType type) const
 {
     BaseQtVersion *version = QtKitAspect::qtVersion(k);
@@ -639,13 +603,12 @@ BuildInfo QmakeBuildConfigurationFactory::createBuildInfo(const Kit *k,
     info.kitId = k->id();
 
     // check if this project is in the source directory:
-    FilePath projectFilePath = FilePath::fromString(projectPath);
-    if (version && version->isInSourceDirectory(projectFilePath)) {
+    if (version && version->isInSourceDirectory(projectPath)) {
         // assemble build directory
-        QString projectDirectory = projectFilePath.toFileInfo().absolutePath();
+        QString projectDirectory = projectPath.toFileInfo().absolutePath();
         QDir qtSourceDir = QDir(version->sourcePath().toString());
         QString relativeProjectPath = qtSourceDir.relativeFilePath(projectDirectory);
-        QString qtBuildDir = version->qmakeProperty("QT_INSTALL_PREFIX");
+        QString qtBuildDir = version->prefix().toString();
         QString absoluteBuildPath = QDir::cleanPath(qtBuildDir + QLatin1Char('/') + relativeProjectPath);
 
         info.buildDirectory = FilePath::fromString(absoluteBuildPath);
@@ -657,7 +620,7 @@ BuildInfo QmakeBuildConfigurationFactory::createBuildInfo(const Kit *k,
     return info;
 }
 
-static QList<BuildConfiguration::BuildType> availableBuildTypes(const BaseQtVersion *version)
+static const QList<BuildConfiguration::BuildType> availableBuildTypes(const BaseQtVersion *version)
 {
     QList<BuildConfiguration::BuildType> types = {BuildConfiguration::Debug,
                                                   BuildConfiguration::Release};
@@ -666,32 +629,25 @@ static QList<BuildConfiguration::BuildType> availableBuildTypes(const BaseQtVers
     return types;
 }
 
-QList<BuildInfo> QmakeBuildConfigurationFactory::availableBuilds(const Target *parent) const
+QList<BuildInfo> QmakeBuildConfigurationFactory::availableBuilds(const Kit *k, const FilePath &projectPath, bool forSetup) const
 {
     QList<BuildInfo> result;
 
-    const QString projectFilePath = parent->project()->projectFilePath().toString();
-
-    foreach (BuildConfiguration::BuildType buildType,
-             availableBuildTypes(QtKitAspect::qtVersion(parent->kit()))) {
-        BuildInfo info = createBuildInfo(parent->kit(), projectFilePath, buildType);
-        info.displayName.clear(); // ask for a name
-        info.buildDirectory.clear(); // This depends on the displayName
-        result << info;
-    }
-
-    return result;
-}
-
-QList<BuildInfo> QmakeBuildConfigurationFactory::availableSetups(const Kit *k, const QString &projectPath) const
-{
-    QList<BuildInfo> result;
     BaseQtVersion *qtVersion = QtKitAspect::qtVersion(k);
-    if (!qtVersion || !qtVersion->isValid())
-        return result;
 
-    foreach (BuildConfiguration::BuildType buildType, availableBuildTypes(qtVersion))
-        result << createBuildInfo(k, projectPath, buildType);
+    if (forSetup && (!qtVersion || !qtVersion->isValid()))
+        return {};
+
+
+    for (BuildConfiguration::BuildType buildType : availableBuildTypes(qtVersion)) {
+        BuildInfo info = createBuildInfo(k, projectPath, buildType);
+        if (!forSetup) {
+            info.displayName.clear(); // ask for a name
+            info.buildDirectory.clear(); // This depends on the displayName
+        }
+        result << info;
+
+    }
 
     return result;
 }
@@ -716,8 +672,8 @@ void QmakeBuildConfiguration::setupBuildEnvironment(Kit *k, Environment &env)
 {
     prependCompilerPathToEnvironment(k, env);
     const BaseQtVersion *qt = QtKitAspect::qtVersion(k);
-    if (qt && !qt->binPath().isEmpty())
-        env.prependOrSetPath(qt->binPath().toString());
+    if (qt && !qt->hostBinPath().isEmpty())
+        env.prependOrSetPath(qt->hostBinPath().toString());
 }
 
 QmakeBuildConfiguration::LastKitState::LastKitState() = default;
