@@ -38,8 +38,7 @@ namespace Internal {
 
 // ---------- CodeGenSettingsPageWidget
 
-CodeGenSettingsPageWidget::CodeGenSettingsPageWidget(QWidget *parent) :
-        QWidget(parent)
+CodeGenSettingsPageWidget::CodeGenSettingsPageWidget()
 {
     m_ui.setupUi(this);
     connect(m_ui.includeQtModuleCheckBox, &QAbstractButton::toggled,
@@ -89,13 +88,17 @@ void CodeGenSettingsPageWidget::setUiEmbedding(int v)
 }
 
 // ---------- CodeGenSettingsPage
-CodeGenSettingsPage::CodeGenSettingsPage(QObject *parent) :
-    Core::IOptionsPage(parent)
+CodeGenSettingsPage::CodeGenSettingsPage()
 {
     m_parameters.fromSettings(Core::ICore::settings());
     setId(Constants::CODEGEN_SETTINGS_PAGE_ID);
-    setDisplayName(QCoreApplication::translate("QtSupport", Constants::CODEGEN_SETTINGS_PAGE_NAME));
+    setDisplayName(QCoreApplication::translate("QtSupport", "Qt Class Generation"));
     setCategory(CppTools::Constants::CPP_SETTINGS_CATEGORY);
+    setDisplayCategory(
+        QCoreApplication::translate("CppTools", CppTools::Constants::CPP_SETTINGS_NAME));
+    setCategoryIcon(Utils::Icon({{":/projectexplorer/images/settingscategory_cpp.png",
+                                  Utils::Theme::PanelTextColorDark}},
+                                Utils::Icon::Tint));
 }
 
 QWidget *CodeGenSettingsPage::widget()
